@@ -1,3 +1,24 @@
+"""
+Copyright (c) 2015-2018 Nitrokey UG
+
+This file is part of libnitrokey.
+
+libnitrokey is free software: you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+any later version.
+
+libnitrokey is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with libnitrokey. If not, see <http://www.gnu.org/licenses/>.
+
+SPDX-License-Identifier: LGPL-3.0
+"""
+
 import pytest
 
 from conftest import skip_if_device_version_lower_than
@@ -143,9 +164,9 @@ def test_password_safe_slot_status(C):
 
 @pytest.mark.aes
 def test_issue_device_locks_on_second_key_generation_in_sequence(C):
-    if is_pro_rtm_07(C) or is_pro_rtm_08(C):
-        pytest.skip("issue to register: device locks up "
-                     "after below commands sequence (reinsertion fixes), skipping for now")
+#    if is_pro_rtm_07(C) or is_pro_rtm_08(C):
+    pytest.skip("issue to register: device locks up "
+                "after below commands sequence (reinsertion fixes), skipping for now")
     assert C.NK_build_aes_key(DefaultPasswords.ADMIN) == DeviceErrorCode.STATUS_OK
     assert C.NK_build_aes_key(DefaultPasswords.ADMIN) == DeviceErrorCode.STATUS_OK
 
