@@ -1,21 +1,24 @@
 /*
  * Author: Copyright (C) Andrzej Surowiec 2012
  *                      Parts Rudolf Boeddeker  Date: 2013-08-13
+ * Copyright (c) 2012-2018 Nitrokey UG
  *
- * This file is part of Nitrokey.
+ * This file is part of Nitrokey App.
  *
- * Nitrokey is free software: you can redistribute it and/or modify
+ * Nitrokey App is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * any later version.
  *
- * Nitrokey is distributed in the hope that it will be useful,
+ * Nitrokey App is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Nitrokey. If not, see <http://www.gnu.org/licenses/>.
+ * along with Nitrokey App. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0
  */
 
 #ifndef MAINWINDOW_H
@@ -54,7 +57,7 @@ public :
   ~MainWindow();
 
 protected:
-  void closeEvent(QCloseEvent *event);
+  void closeEvent(QCloseEvent *event) override;
 
 private:
   Q_DISABLE_COPY(MainWindow);
@@ -69,6 +72,7 @@ private:
   const unsigned char TOTP_SlotCount;
   DebugDialog *debug;
 
+  void keyPressEvent(QKeyEvent *keyevent) override;
 
   bool validate_raw_secret(const char *secret) const;
   void initialTimeReset();
@@ -98,7 +102,9 @@ private:
 
 public slots:
   void startAboutDialog();
-  void startConfiguration();
+  void startHelpAction();
+  void startConfiguration(bool changeTab = true);
+  void startConfigurationMain();
   void PWS_Clicked_EnablePWSAccess();
 
   int factoryResetAction();
@@ -126,8 +132,11 @@ signals:
   void ShortOperationEnds();
   void OTP_slot_write(int slot_no, bool isHOTP);
   void DebugData(QString msg);
+  void LongOperationStart();
 
 private slots:
+  void manageStartPage();
+  void on_longOperationStart();
   void on_KeepDeviceOnline();
   void on_DeviceConnected();
   void on_DeviceDisconnected();
@@ -135,6 +144,7 @@ private slots:
   void on_enableUserPasswordCheckBox_clicked(bool checked);
 
   void resizeMin();
+  void ready();
   void checkConnection();
   void storage_check_symlink();
 
@@ -162,6 +172,7 @@ private slots:
   void on_checkBox_toggled(bool checked);
 
   void startStickDebug();
+  void load_settings_page();
 
   void on_PWS_ButtonClearSlot_clicked();
   void on_PWS_ComboBoxSelectSlot_currentIndexChanged(int index);
@@ -178,6 +189,16 @@ private slots:
   void on_PWS_EditPassword_textChanged(const QString &arg1);
 
 
+  void on_btn_writeSettings_clicked();
+
+  void on_btn_select_debug_file_path_clicked();
+
+  void on_PWS_Lock_clicked();
+
+  void on_btn_copyToClipboard_clicked();
+
+  void on_btn_select_debug_console_clicked();
+
 public:
   void generateOTPConfig(OTPSlot *slot);
   unsigned int get_supported_secret_length_base32() const;
@@ -208,6 +229,8 @@ private:
   unsigned int roundToNextMultiple(const int number, const int multipleOf) const;
 
   QString getOTPSecretCleaned(QString secret_input);
+
+  void make_UI_enabled(bool enabled);
 };
 
 class utf8FieldLengthValidator : public QValidator {
